/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// constants
const ffIError = Components.interfaces.ffIError;

const FF_TIME_NIGHT = 3; // time that night switches to day
const FF_TIME_DAY = 15;  // time that day switches to night

//global forecastfox object
var gForecastfox = null;

function ffLoad()
{
  gForecastfox = new Forecastfox();
  gForecastfox.start();
};

function ffUnload()
{
  gForecastfox.stop();
  gForecastfox = null;
};

function ffResize(event)
{
  //XXX bug 9355
  //gForecastfox.onResize(evenft);
};

///////////////////////
// Forecastfox Object
function Forecastfox()
{
  return this;
};

Forecastfox.prototype = {
  _manager: null,
  _domain: "forecastfox.",
  _branch: null,
  _bundle: null,
  _prefixes: null,
  
  start: function()
  {
    //get our XPCOM manager service
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    this._manager = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager);
    this._branch = pbs.getBranch(this._domain);
    this._bundle = document.getElementById("ff-bundle");  
    
    //setup object
    this._prefixes = new Object();
    this._prefixes["cc"] = "current";
    this._prefixes["dayf"] = "forecast";
    this._prefixes["dayt"] = "forecast";

    //add observer to forecastfox:update
    obs.addObserver(this, "forecastfox:manager", false);
    
    //add resize listener
    //XXX bug 9355
    //window.addEventListener("resize", ffResize, false);

    //setup display - give window a chance to display before loading
    if (this._manager.firstTime)
      setTimeout("gForecastfox._manager.load(false);", 10);
    else
      this._update(null, "");
  },

  stop: function()
  {
    try {
      //remove observers
      var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obs.removeObserver(this, "forecastfox:manager");
      obs = null;
    } catch(e) {};

    //destroy variables
    this._manager = null;
    this._domain = null;
    this._branch = null;
    this._bundle = null;
    this._prefixes = null;
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    var method = null;
    var data = null;
    if (aData.indexOf(",") != -1) {
      method = aData.substring(0, aData.indexOf(","));
      data = aData.substring(aData.indexOf(",") + 1, aData.length);
    } else
      method = aData.substring(0, aData.length);
    switch (method) {
      case "move":
      case "profile":
        this._moveBox();
        break;
      case "enable":
        this._update(null, "end");
        break;
      case "data":
        var group = this._prefixes[data];
        this._update(group, "end");
        break;
      default:
        this._update(data, method);
        break;
    }
  },

  hideTooltip: function(aNode)
  {
    if (!aNode.hasAttribute("tooltip"))
      return;
      
    var tooltip = document.getElementById(aNode.getAttribute("tooltip"));
    tooltip.hidePopup();
  },
  
  populateProfiles: function()
  {
    var profmenu = document.getElementById("ff-profs-popup");
    var keys = this._manager.profiles.getProfiles({});
    var item;
        
	//remove old profiles
    while (profmenu.hasChildNodes())
      profmenu.removeChild(profmenu.lastChild);
    
    //set none
    if (keys.length == 0) {
      item = document.createElement("menuitem");
      item.setAttribute("label", this._bundle.getString("ff.profiles.none"));
      item.setAttribute("disabled", true);
      profmenu.appendChild(item);
      return;
    };
    
    //set for one or more
    for (var x=0; x<keys.length; x++) {
      item = document.createElement("menuitem");
      item.setAttribute("label", keys[x]);
      item.setAttribute("oncommand", "gForecastfox.switchProfile(this)"); 
      item.setAttribute("type", "radio");
      item.setAttribute("name", "profile");     
      if (keys[x] == this._branch.getCharPref("profile.current"))
        item.setAttribute("checked", "true");
      profmenu.appendChild(item);
    }
  },
  
  switchProfile: function(aItem)
  {
    //hide popup
    document.getElementById("ff-profs-popup").hidePopup();
    
    //get selected profile
    var profile = aItem.getAttribute("label");
    
    //set if not current
    if (profile != this._branch.getCharPref("profile.current"))
      this._branch.setCharPref("profile.current", profile);
  },

  openLink: function(aEvent, aUnits, aPartner)
  {
    // supply an aOpenEvent and it will be used in the openUILink instead of the aEvent
    var url = aEvent.explicitOriginalTarget.getAttribute("url");
    if (aPartner)
      url = url + "?partner=" + this._manager.partner;
    if (aUnits) {
      if (this._branch.getIntPref("general.units") > 1)
 	    url += "&metric=1";
 	  else
 	    url += "&metric=0";
    }
 	
    try {
      openUILink(url, aEvent);
    }
    catch (e) {
      window._content.document.location = url;
    }
  },
  
  _moveBox: function()
  {
    //check if box is in the right location
    var toolbar = document.getElementById(this._branch.getCharPref("general.bar"));
    var box = document.getElementById("ff-box");
    var position = this._branch.getIntPref("general.position");
    
    //if toolbar doesn't exist move to default location
    if (!toolbar) {
      this._branch.setCharPref("general.bar", "status-bar");
      this._branch.setIntPref("general.position", -1);
      return;
    };
    
    // bail if it is in the right place
    if (this._indexOf(toolbar,box) == position)
      return;

    //remove us from parent
    box.parentNode.removeChild(box);
    this._insertAtIndex(toolbar, box, position);
  },
  
  _indexOf: function(aParent, aChild)
  {
    // -1 if it does not exist
    var children = aParent.childNodes;
    for (var x=0; x<children.length; x++)
      if (children[x] == aChild)
        return (x == children.length - 1) ? -1 : x;
   
    return null;
  },
  
  _insertAtIndex: function(aParent, aChild, aIndex)
  {
    var children = aParent.childNodes;
    if ((children.length == 0) || (aIndex >= children.length) || (aIndex < 0))
      aParent.appendChild(aChild);
    else
      aParent.insertBefore(aChild, children[aIndex]);
  },
  
  _update: function(aGroup, aMethod)
  {
    var error;
    
    //update all
    if (!aGroup) {
      //do current
      error = this._manager.getError("current");
      if (error.severity != ffIError.SEVERITY_INFO)
        this._update("current", "error");
      else
        this._update("current", "end");
      //do forecast
      error = this._manager.getError("forecast");
      if (error.severity != ffIError.SEVERITY_INFO)
        this._update("forecast", "error");
      else
        this._update("forecast", "end");
      this._borders();
      return;        
    };
    
    //move box
    this._moveBox();
    
    //remove progress and errors
    this._removeError(aGroup);
    this._removeProgress(aGroup);
    
    //now do update
    switch (aMethod) {
      case "start":
        this._showProgress(aGroup);
        break;
      case "error":
        this._showError(aGroup);
        break;
      case "end":
      default:
        if (aGroup == "current")
          this._showCurrent();
        else
          this._showForecast();
        break;
    };
    
    this._borders();
  },

  _removeError: function(aItem)
  {
    var item = this._getItem(aItem, "error"); 
    
    //remove error  
    this._removeTooltip(item);
    item.removeAttribute("value");
    item.setAttribute("collapsed", true);
    
    //show panel
    item.parentNode.setAttribute("selectedIndex", "0");    
  },
  
  _removeProgress: function(aItem)
  {
    var item = this._getItem(aItem, "progress");   
    
    //remove progress
    this._removeTooltip(item);
    item.setAttribute("mode", "determined");
    item.setAttribute("value", "0");
    item.setAttribute("collapsed", "true");
    
    //show panel
    item.parentNode.setAttribute("selectedIndex", "0");
  },
  
  _removeTooltip: function(aItem)
  {
    //remove tooltiptext
    if (aItem.hasAttribute("tooltiptext")) {
      aItem.removeAttribute("tooltiptext");
    };
    
    //remove tooltip elements
    if (aItem.hasAttribute("tooltip")) {
      var tooltip = document.getElementById(aItem.getAttribute("tooltip"));
      tooltip.parentNode.removeChild(tooltip);
      aItem.removeAttribute("tooltip");   
    } 
  },

  _showError: function(aItem)
  {
    var item = this._getItem(aItem, "error");   
    
    //set error
    var error = this._manager.getError(aItem);
    item.setAttribute("value", error.description);
    item.setAttribute("tooltiptext", error.tooltip);

    //show panel
    item.removeAttribute("collapsed");
    item.parentNode.setAttribute("selectedIndex", "1");      
  },
  
  _showProgress: function(aItem)
  {
    var item = this._getItem(aItem, "progress");   
    
    //set progress
    item.removeAttribute("value");
    item.setAttribute("mode", "undetermined");
    item.setAttribute("tooltiptext", this._bundle.getString("ff.tooltip.loading." + aItem));    
    
    //show panel
    item.removeAttribute("collapsed");
    item.parentNode.setAttribute("selectedIndex", "2");  
  },

  _showCurrent: function()
  {
    var current = this._getItem("current", "panel");
    var radar = this._getItem("radar", "panel");
    var swa = this._getItem("swa", "panel");
    var deck = this._getItem("current", "deck");
       
    //remove tooltip and attributes
    this._removeTooltip(current);
    current.removeAttribute("src");
    current.removeAttribute("label");
    this._removeTooltip(radar);
    radar.removeAttribute("src");
    radar.removeAttribute("label");
    this._removeTooltip(swa);
    swa.removeAttribute("src");
    swa.removeAttribute("label");
        
    //do current
    if (!this._branch.getBoolPref("cc.panel.enabled"))
      current.setAttribute("collapsed", "true");
    else {
      this._setPanel(current, "cc", null, null);
      current.setAttribute("url", this._manager.parser.getValue("current", "url", null));
      current.removeAttribute("collapsed"); 
    };
    
    //do radar
    if (!this._branch.getBoolPref("radar.panel.enabled") || !this._branch.getBoolPref("cc.panel.enabled"))    
      radar.setAttribute("collapsed", "true");
    else {
      this._setPanel(radar, "radar", "radar", null);
      
      radar.setAttribute("url", this._manager.parser.getValue("radar", "url", null));
      
      // only display the radar node if there is radar
      if (this._branch.getCharPref("radar.cache") != "")
        radar.removeAttribute("collapsed");
      else
        radar.setAttribute("collapsed", true);
    };
    
    //do swa
    if (!this._branch.getBoolPref("swa.panel.enabled") || this._manager.parser.getValue("swa", "active", null) != "true")    
      swa.setAttribute("collapsed", "true");
    else {
      this._setPanel(swa, "swa", "swa", null);
      
      swa.setAttribute("url", this._manager.parser.getValue("swa", "url", null));
      swa.removeAttribute("collapsed");       
    };
    
    //display
    deck.setAttribute("selectedIndex", "0");
  },
  
  _showForecast: function()
  {
    var today = this._getItem("today", "panel");
    var extended = this._getItem("forecast", "panel");
    var deck = this._getItem("forecast", "deck");
       
    //remove tooltip and attributes
    while (today.hasChildNodes()) {
      this._removeTooltip(today.lastChild);
      today.lastChild.removeAttribute("src");
      today.lastChild.removeAttribute("label");
      today.removeChild(today.lastChild);
    };
    while (extended.hasChildNodes()) {
      this._removeTooltip(extended.lastChild);
      extended.lastChild.removeAttribute("src");
      extended.lastChild.removeAttribute("label");
      extended.removeChild(extended.lastChild);
    };        
    
    //set url for popup menu since it is the same for all forecast days
    var menu = document.getElementById("ff-popup-forecast");
    menu.setAttribute("url", this._manager.parser.getValue("days", "url", null));
    
    //do today
    if (!this._branch.getBoolPref("dayt.panel.enabled") || (this._branch.getIntPref("dayt.panel.mode") < 0))
      today.setAttribute("collapsed", "true");
    else
      this._updateToday(today);
  
    //do extended
    if (!this._branch.getBoolPref("dayf.panel.enabled") || (this._branch.getIntPref("dayf.panel.days") == 0)) 
      extended.setAttribute("collapsed", "true");
    else
      this._updateExtended(extended);
    
    //display
    deck.setAttribute("selectedIndex", "0");  
  },
  
  _updateToday: function(aNode)
  {	
    var mode = this._branch.getIntPref("dayt.panel.mode");
    var sw_mode = this._branch.getIntPref("dayt.panel.switch");
    var panels = (mode == 2) ? 2 : 1;
    var cHours = (new Date()).getHours();
    var day_index = 0;
  	
    while (this._shouldIncDay(day_index))
      day_index++;
  	
    for (var x = 0; x < panels; x++) {
  	
      if (cHours <= FF_TIME_NIGHT || cHours >= FF_TIME_DAY) {
        if (mode == 0 && sw_mode == 1)
          day_index++;
        else if (mode == 2)
          panels--;
      };

      if (this._shouldCreatePanel("today", "days", x))
        this._createPanel("today", aNode, "days", day_index);
      else if (this._shouldCreatePanel("today", "nights", x))
        this._createPanel("today", aNode, "nights", day_index);
    }
  	
    // show the box!
    aNode.removeAttribute("collapsed");
  },
  
  _updateExtended: function(aNode)
  {
	var days = this._branch.getIntPref("dayf.panel.days");
    var new_panel = false;
    var cHours    = (new Date()).getHours();
    var day_index = 1;
    var day_add   = 0;

    while (this._shouldIncDay(day_add))
      day_add++;
      
    day_index += day_add;
        
    for (var x = 0; x < days; day_index++) {
      // finish if we went through too many days
      if (day_index > 9)
        break;

      // create the corresponding nodes
      if (this._shouldCreatePanel("extended", "days", day_index))
        new_panel = this._createPanel("extended", aNode, "days", day_index);
        
      if (this._shouldCreatePanel("extended", "nights", day_index))
        new_panel = this._createPanel("extended", aNode, "nights", day_index);
      
      // increase x if we made a new one
      if (new_panel)
        x++;
      
      new_panel = false;
    }
    
    aNode.removeAttribute("collapsed");
  },
  
  _shouldIncDay: function(aIndex)
  {
    var cHours = (new Date()).getHours();
    return ((cHours < 12 && !this._hasData(aIndex, cHours)) && 
    		(FF_TIME_NIGHT < cHours && cHours < FF_TIME_DAY));
  },
  
  _hasData: function(aIndex, aHours)
  {
    return ((this._manager.parser.getValue("days", "t", aIndex) != "N/A") ||
    	    (this._manager.parser.getValue("days", "tlong", aIndex) != "N/A"));
  },
  
  _shouldCreatePanel: function(aType, aPart, aIndex) 	 
  {
    var mode, sw_mode, cHours, t_mode, t_sw_mode, t_on;

    cHours = (new Date()).getHours();
    
    switch (aType) {
      case "today":
        mode = this._branch.getIntPref("dayt.panel.mode");
    	sw_mode = this._branch.getIntPref("dayt.panel.switch");
    	
        switch (aPart) {
          case "days":
            if (cHours > FF_TIME_NIGHT && cHours < FF_TIME_DAY)
              return ((mode == 0) || (mode == 2 && aIndex == 0));
            else
              return ((mode == 0 && sw_mode == 1) || (mode == 2 && aIndex == 2));
          case "nights":
            if (mode == 1)
              return true;
            else if (cHours > FF_TIME_NIGHT && cHours < FF_TIME_DAY)
              return (mode == 2 && aIndex == 1);
            else
              return ((mode == 0 && sw_mode == 0) || (mode == 2));
        };
        break;
      case "extended":
        mode = this._branch.getIntPref("dayf.panel.mode");
        t_mode = this._branch.getIntPref("dayt.panel.mode");
        t_sw_mode = this._branch.getIntPref("dayt.panel.switch");
        t_on = this._branch.getBoolPref("dayt.panel.enabled");

        switch (aPart) {
          
          case "nights":
            return ((mode == 1) || (mode == 2));
          case "days":
            // if we're displaying days or days and nights
            if (mode == 0 || mode == 2) {
              // If its between the "days" hours
              if (FF_TIME_NIGHT < cHours && cHours < FF_TIME_DAY)
                return true;
              // If its between the "nights" hours
              else
                return ((t_mode != 0) || (!t_on || (aIndex != 1 && t_sw_mode == 1)) || (t_sw_mode != 1));
            }
            else
              return false;
        };
        break;
    };
  },
  
  _createPanel: function(aGroup, aParent, aPart, aDay)
  {
    var node = document.createElement("statusbarpanel");
    var prefix = (aGroup == "extended") ? "dayf" : "dayt";
    
    //set context menu on node
    node.setAttribute("context","ff-popup");
    node.setAttribute("id", "forecast" + aPart + aDay);
    node.setAttribute("class", "ff-panel");
    node.setAttribute("onmouseout", "gForecastfox.hideTooltip(this);");
    node.setAttribute("url", this._manager.parser.getValue(aPart, "url", aDay));

    //set panel and append
    this._setPanel(node, prefix, aPart, aDay);
    aParent.appendChild(node);
    
    // return that one was made
    return true;
  },
      
  _getItem: function(aItem, aType)
  {
    var item = null;
    item = document.getElementById("ff-" + aItem + "-" + aType);
    return item;  
  },
  
  _setPanel: function(aNode, aPrefix, aGroup, aIndex)
  {
    //if group isn't provided get from prefix
    if (!aGroup)
      aGroup = this._prefixes[aPrefix];
      
    //set tooltip
    if (this._branch.getBoolPref(aPrefix + ".tooltip.enabled"))
      this._setTooltip(aNode, aPrefix, aGroup, aIndex);
       
    //set display  
    var icon = "chrome://forecastfox/skin/images/small/" + this._manager.parser.getValue(aGroup, "icon", aIndex) + ".png";
    var label = this._manager.parser.setLabel(this._branch.getComplexValue(aPrefix + ".panel.label",Components.interfaces.nsIPrefLocalizedString).data, aGroup, aIndex);
    switch (this._branch.getIntPref(aPrefix + ".panel.display")) {
       case 0:
       default:
       aNode.setAttribute("src", icon);
       break;
     case 1:
       aNode.setAttribute("label", label);
       break;
     case 2:
       aNode.setAttribute("src", icon);
       aNode.setAttribute("label", label); 
       break;
    }
  },
  
  _setTooltip: function(aNode, aPrefix, aGroup, aIndex)
  {
    //if group isn't provided get from prefix
    if (!aGroup)
      aGroup = this._prefixes[aPrefix];
     
    var lblparent, imgparent;
    
    //create empty tip
    var parent = document.getElementById("ff-popupset");
    var tooltip = document.createElement("tooltip");
    tooltip.setAttribute("id", aNode.getAttribute("id") + "_tip");
    tooltip.setAttribute("noautohide", "true");
    tooltip.setAttribute("align", "center");
            
    //create an image
    var icon = "chrome://forecastfox/skin/images/large/" + this._manager.parser.getValue(aGroup, "icon", aIndex) + ".png";    
    var image = document.createElement("image");
    if (aPrefix == "radar") {
      var fileUrl = this._getFileURL(this._branch.getCharPref("radar.cache"));
      this._expireUrlInCache(fileUrl);
      image.setAttribute("validate", "always");
      image.setAttribute("src", fileUrl);
      image.setAttribute("class", "ff-radaricon");
    } else {
      image.setAttribute("src", icon);
      image.setAttribute("class", "ff-tipicon");
    };
        
    //set image 
    switch (this._branch.getIntPref(aPrefix + ".tooltip.display")) {
      case 0:
        tooltip.appendChild(image);
        break;
      case 1:
        tooltip.setAttribute("orient", "vertical");
        lblparent = tooltip;
        break;
      case 2:
        imgparent = document.createElement("vbox");
        imgparent.appendChild(image);
        lblparent = document.createElement("vbox");
        tooltip.setAttribute("orient", "horizontal");
        tooltip.appendChild(imgparent);
        tooltip.appendChild(lblparent);                             
        break;
    };
    
    //set label
    if (this._branch.getIntPref(aPrefix + ".tooltip.display") > 0) {
      var lbls = this._branch.getComplexValue(aPrefix + ".tooltip.label",Components.interfaces.nsIPrefLocalizedString).data;
      lbls = this._manager.parser.setLabel(lbls, aGroup, aIndex);   
      lbls = lbls.split("/n");  
      for (var i=0; i<lbls.length; ++i) {
        var lbl = document.createElement("label");
        lbl.setAttribute("value", lbls[i]);
        lblparent.appendChild(lbl);
      };
    };

    //append new tooltip
    parent.appendChild(tooltip);    

    //associate tooltip with panel    
    aNode.setAttribute("tooltip", tooltip.getAttribute("id"));
  },
  
  _borders: function()
  {
    //get elements
    var spacer = document.getElementById("ff-spacer");
    var today = this._getItem("today", "panel");
    var extended = this._getItem("forecast", "panel");
    var radar = this._getItem("radar", "panel");
    var swa = this._getItem("swa", "panel");
    var current = this._getItem("current", "panel");
    var fDeck = this._getItem("forecast", "deck");
    var cDeck = this._getItem("current", "deck");
    var showing = new Object();
    
    //determine what is showing
    showing["cDeck"] = (cDeck.getAttribute("selectedIndex") != "0") ? true : false;
    showing["fDeck"] = (fDeck.getAttribute("selectedIndex") != "0") ? true : false;
    showing["current"] = (this._branch.getBoolPref("cc.panel.enabled") && !current.hasAttribute("collapsed"));
    showing["radar"] = (this._branch.getBoolPref("radar.panel.enabled") && !radar.hasAttribute("collapsed"));
    showing["swa"] = (this._branch.getBoolPref("swa.panel.enabled") && !swa.hasAttribute("collapsed"));
    showing["today"] = (this._branch.getBoolPref("dayt.panel.enabled") && today.hasChildNodes() && !today.hasAttribute("collapsed"));
    showing["extended"] = (this._branch.getBoolPref("dayf.panel.enabled") && extended.hasChildNodes() && !extended.hasAttribute("collapsed"));
    
    //show-hide spacer
    var cShowing = (showing["cDeck"] || showing["current"] || showing["radar"] || showing["swa"]);
    var fShowing = (showing["fDeck"] || showing["today"] || showing["extended"]);
    if (cShowing && fShowing)
      spacer.removeAttribute("collapsed");
    else
      spacer.setAttribute("collapsed", "true");
        
    //remove old first-last attribute
    var box = document.getElementById("ff-box");
    var panels = box.getElementsByTagName("statusbarpanel");
    for (var x=0; x<panels.length; x++) {
      panels[x].removeAttribute("ff-first");
      panels[x].removeAttribute("ff-last");
    }
    
    //set first attribute
    if (!showing["cDeck"] && showing["swa"])
      swa.setAttribute("ff-first", "true");    
    else if (!showing["cDeck"] && showing["radar"])
      radar.setAttribute("ff-first", "true");
    else if (!showing["cDeck"] && showing["current"])
      current.setAttribute("ff-first", "true");
    else if (!showing["fDeck"] && showing["today"])
      today.firstChild.setAttribute("ff-first", "true");
    else if (!showing["fDeck"] && showing["exteneded"])
      extended.firstChild.setAttribute("ff-first", "true");

    //set last attribute
    if (!showing["fDeck"] && showing["extended"])
      extended.lastChild.setAttribute("ff-last", "true");
    else if (!showing["fDeck"] && showing["today"])
      today.lastChild.setAttribute("ff-last", "true");
    else if (!showing["cDeck"] && showing["current"])
      current.setAttribute("ff-last", "true");
    else if (!showing["cDeck"] && showing["radar"])
      radar.setAttribute("ff-last", "true");
    else if (!showing["cDeck"] && showing["swa"])
      swa.setAttribute("ff-last", "true");      
  },
  
  _getFileURL: function(aName)
  {
    var dirs = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
    var file = dirs.get("ProfD", Components.interfaces.nsIFile);
    
    //get file
    file.append("forecastfox");     
    file.append(aName);
       
    //convert to a URI and return spec
    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    return ios.newFileURI(file).spec;  
  },
  
  _expireUrlInCache: function(aUrl)
  {
    var cacheServ = Components.classes["@mozilla.org/network/cache-service;1"].getService(Components.interfaces.nsICacheService);
    var session = cacheServ.createSession("image", Components.interfaces.nsICache.ACCESS_READ_WRITE, true);
    try {
      var entry = session.openCacheEntry(aUrl, Components.interfaces.nsICache.ACCESS_READ_WRITE, true);
      entry.setExpirationTime(0);
    } catch (e) { }
  },
            
  //XXX this will be reworked  
  onResize: function(event)
  {
    // Collapses the deck if the toolbar is wider than the window and the window is not maximized.
    this._box.removeAttribute("collapsed");
    
    var toolbar = this._box.parentNode;
    var twidth  = 0;
    
    for (var x = 0; x < toolbar.childNodes.length; x++)
      twidth += toolbar.childNodes[x].boxObject.width;
    
    if (twidth > window.innerWidth && window.windowState != Components.interfaces.nsIDOMChromeWindow.STATE_MAXIMIZED)
      this._box.setAttribute("collapsed", true);
  }
}
